//
//  AppDelegate.m
//  OpenCLDemo
//
//  Created by Torsten Kammer on 26.10.11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "AppDelegate.h"

@interface AppDelegate()
{
	cl_device_id device;
	cl_context context;
	cl_command_queue queue;
	cl_program program;
	cl_kernel kernel;
}

@end

@implementation AppDelegate

@synthesize window = _window;
@synthesize numElements;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	clGetDeviceIDs(NULL, CL_DEVICE_TYPE_GPU, 1, &device, NULL);
	
	context = clCreateContext(NULL, // Platform
							  1, &device, // Devices
							  clLogMessagesToStderrAPPLE, // Error callback
							  NULL, // User data
							  NULL); // Error out
	queue = clCreateCommandQueue(context, // Context
								 device, // Device
								 0, // Options
								 NULL); // Error out

	NSString *programSource = [NSString stringWithContentsOfURL:[[NSBundle mainBundle] URLForResource:@"add" withExtension:@"cl"] encoding:NSUTF8StringEncoding error:nil];
	const char *utf8Source = [programSource UTF8String];
	
	program = clCreateProgramWithSource(context, 1, &utf8Source, NULL, NULL);
	
	clBuildProgram(program, 0, NULL, NULL, NULL, NULL);
	
	
	kernel = clCreateKernel(program, "add", NULL);

}

- (void)dealloc
{
	clReleaseKernel(kernel);
	clReleaseProgram(program);
	clReleaseCommandQueue(queue);
	clReleaseContext(context);
}

- (IBAction)calculate:(id)sender
{
	NSUInteger count = self.numElements;
	
	float *a = malloc(sizeof(float) * count);
	float *b = malloc(sizeof(float) * count);
	for (NSUInteger i = 0; i < count; i++)
	{
		a[i] = b[i] = (float) i;
	}

	cl_mem bufferA = clCreateBuffer(context, // CL Context
									CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, // Options
									count*sizeof(float), // Size
									a, // Data to copy
									NULL); // Error out (using callback instead)
	
	cl_mem bufferB = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, count*sizeof(float), b, NULL);
	cl_mem bufferC = clCreateBuffer(context, CL_MEM_WRITE_ONLY, count*sizeof(float), NULL, NULL);


	free(a);
	free(b);

	clSetKernelArg(kernel, 0, sizeof(cl_mem), &bufferA);
	clSetKernelArg(kernel, 1, sizeof(cl_mem), &bufferB);
	clSetKernelArg(kernel, 2, sizeof(cl_mem), &bufferC);
	
	size_t workOffset[1] = { 0 };
	size_t workSize[1] = { count };
	clEnqueueNDRangeKernel(queue, kernel, 1, workOffset, workSize, NULL, 0, NULL, NULL);


	float *c = calloc(sizeof(float), count);
	clEnqueueReadBuffer(queue, // Command Queue
						bufferC, // Buffer
						CL_TRUE, // Block until read is done?
						0, // Offset
						count*sizeof(float), // Num Bytes
						c, // Output
						0, // # events to wait for
						NULL, // events to wait for
						NULL); // output event
	
	for (NSUInteger i = 0; i < count; i++)
	{
		printf("%f ", c[i]);
	}
	
	free(c);
	
	clReleaseMemObject(bufferA);
	clReleaseMemObject(bufferB);
	clReleaseMemObject(bufferC);
}

@end
